@@ -1,6 +1,6 @@
 package Apache::Voodoo::Session::Instance;
 
-$VERSION = "3.0002";
+$VERSION = "3.0100";
 
 use strict;
 use warnings;
@@ -19,6 +19,8 @@ sub new {
 
 	$self->{id} = $session->{_session_id};
 
+	$self->{connected} = 1;
+
 	return $self;
 }
 
@@ -30,7 +32,7 @@ sub has_expired {
 	my $self    = shift;
 	my $timeout = shift;
 
-	if ($timeout > 0 && $self->{session}->{timestamp} < (time - ($timeout*60))) {
+	if ($timeout > 0 && $self->{session}->{_session_timestamp} < (time - ($timeout*60))) {
 		return 1;
 	}
 	else {
@@ -41,7 +43,7 @@ sub has_expired {
 sub touch {
 	my $self = shift;
 
-	$self->{session}->{timestamp} = time;
+	$self->{session}->{_session_timestamp} = time;
 }
 
 sub disconnect {
