@@ -7,7 +7,7 @@
 ################################################################################
 package Apache::Voodoo::Table;
 
-$VERSION = "3.0002";
+$VERSION = "3.0100";
 
 use strict;
 use warnings;
@@ -24,9 +24,43 @@ sub new {
 
 	bless $self, $class;
 
-
 	$self->set_configuration(shift);
 
+	$self->{'list_param_parser'} = sub {
+		my $self   = shift;
+		my $dbh    = shift;
+		my $params = shift;
+
+		my @fields = @{$self->{'columns'}};
+		if ($self->{'references'}) {
+			foreach my $join (@{$self->{'references'}}) {
+				foreach (@{$join->{'columns'}}) {
+					push(@fields,"$join->{'table'}.$_");
+				}
+			}
+		}
+
+		my @search;
+		foreach my $field (@fields) {
+			my $s = 'search_'   .$field;
+			my $o = 'search_op_'.$field;
+
+			next unless defined($params->{$s});
+
+			if (defined($params->{$o})) {
+				push(@search,[$field,$params->{$o},$params->{$s}]);
+			}
+			elsif ($params->{$s} =~ /^\d+$/) {
+				push(@search,[$field,'=',$params->{$s}]);
+			}
+			else {
+				push(@search,[$field,'like',$params->{$s}]);
+			}
+		}
+
+		return @search;
+	};
+
 	return $self;
 }
 
@@ -72,6 +106,7 @@ sub set_configuration {
 			Apache::Voodoo::Exception::RunTime->throw($@);
 	}
 
+	$self->{'column_names'} = {};
 	while (my ($name,$conf) = each %{$conf->{'columns'}}) {
 		if (defined($conf->{'multiple'})) {
 			push(@errors,"Column $name allows multiple values but Apache::Voodoo::Table can't handle that currently.");
@@ -84,6 +119,7 @@ sub set_configuration {
 		# keep a local list of column names for query construction.
 		if (defined($self->{'pkey'}) && $name ne $self->{'pkey'}) {
 			push(@{$self->{'columns'}},$name);
+			$self->{'column_names'}->{$self->{'table'}.'.'.$name} = 1;
 		}
 
 		if ($conf->{'type'} eq "date") { push(@{$self->{dates}},$name); }
@@ -131,39 +167,72 @@ sub set_configuration {
 		$self->{'list_search'}->{$_->[1]} = 1;
 	}
 
+	if ($conf->{'list_options'}->{'group_by'}) {
+		$self->{'group_by'} = $conf->{'list_options'}->{'group_by'};
+		$self->{'group_by'} = $conf->{'table'}.".".$self->{'group_by'} unless ($self->{'group_by'} =~ /\./);
+	}
+
+	$self->{'joins'}      = [];
+	$self->{'list_joins'} = [];
+	$self->{'view_joins'} = [];
+
 	if (ref($conf->{'joins'}) eq "ARRAY") {
-		foreach (@{$conf->{'joins'}}) {
-			push(@{$self->{'joins'}},
+		foreach my $j (@{$conf->{'joins'}}) {
+			$j->{'columns'} ||= [];
+			
+			foreach (@{$j->{'columns'}}) {
+				$self->{'column_names'}->{$j->{'table'}.'.'.$_} = 1;
+			}
+
+			my $context = lc($j->{'context'}) || '';
+			$context = ($context =~ /^(list|view)$/i)?$context."_":'';
+
+			push(@{$self->{$context.'joins'}},
 				{
-					table   => $_->{table},
-					pkey    => $_->{primary_key},
-					fkey    => $_->{foreign_key},
-					columns => $_->{columns} || []
+					table     => $j->{'table'},
+					type      => $j->{'type'} || 'LEFT',
+					pkey      => $j->{'primary_key'},
+					fkey      => $j->{'foreign_key'},
+					columns   => $j->{'columns'},
+					extra     => $j->{'extra'}
 				}
 			);
 		}
 	}
 
-	$self->{'pager'} = Apache::Voodoo::Pager->new();
-	# setup the pagination options
-	$self->{'pager'}->set_configuration(
-		'count'   => 40,
-		'window'  => 10,
-		'persist' => [ 
-			'pattern',
-			'limit',
-			'sort',
-			'last_sort',
-			'desc',
-			@{$conf->{'list_options'}->{'persist'} || []}
-		]
-	);
+	if ($conf->{'pager'}) {
+		$self->{'pager'} = $conf->{'pager'};
+	}
+	else {
+		$self->{'pager'} = Apache::Voodoo::Pager->new();
+		# setup the pagination options
+		$self->{'pager'}->set_configuration(
+			'count'   => 40,
+			'window'  => 10,
+			'persist' => [ 
+				'pattern',
+				'limit',
+				'sort',
+				'last_sort',
+				'desc',
+				@{$conf->{'list_options'}->{'persist'} || []}
+			]
+		);
+	}
 
 	if (@errors) {
 		Apache::Voodoo::Exception::RunTime::BadConfig->throw("Configuration Errors:\n\t".join("\n\t",@errors));
 	}
 }
 
+sub table {
+	my $self = shift;
+	if ($_[0]) {
+		$self->{'table'} = $_[0];
+	}
+	return $self->{'table'};
+}
+
 sub success {
 	my $self = shift;
 
@@ -179,6 +248,15 @@ sub edit_details {
 	return $self->{'edit_details'} || [];
 }
 
+sub add_details {
+	my $self = shift;
+
+	# if there wasn't a successful add, then there's no details :)
+	return unless $self->{'success'};
+
+	return $self->{'add_details'} || [];
+}
+
 sub add_insert_callback {
 	my $self    = shift;
 	my $sub_ref = shift;
@@ -193,6 +271,123 @@ sub add_update_callback {
 	push(@{$self->{'update_callbacks'}},$sub_ref);
 }
 
+sub list_param_parser {
+	my $self    = shift;
+	my $sub_ref = shift;
+
+	$self->{'list_param_parser'} = $sub_ref;
+}
+
+sub validate_add {
+	my $self   = shift;
+	my $p      = shift;
+
+	my $dbh    = $p->{'dbh'};
+	my $params = $p->{'params'};
+
+	my $errors = {};
+
+	# call each of the insert callbacks
+	foreach (@{$self->{'insert_callbacks'}}) {
+		my $callback_errors = $_->($dbh,$params);
+		@{$errors}{keys %{$callback_errors}} = values %{$callback_errors};
+	}
+
+	# do all the normal parameter checking
+	my ($values,$e) = $self->{valid}->validate($params);
+
+	if (defined($e)) {
+		# copy the errors from the process_params
+		$errors = { %{$errors}, %{$e} };
+	}
+
+	# check to see if the user supplied primary key (optional) is unique
+	if ($self->{'pkey_user_supplied'}) {
+		if ($params->{$self->{'pkey'}} =~ /$self->{'pkey_regexp'}/) {
+			my $res = $dbh->selectall_arrayref("
+				SELECT 
+					1 
+				FROM 
+					$self->{'table'} 
+				WHERE 
+					$self->{'pkey'} = ?",
+				undef,
+				$params->{$self->{'pkey'}} );
+
+			if ($res->[0]->[0] == 1) {
+				$errors->{'DUP_'.$self->{'pkey'}} = 1;
+			}
+		}
+		else {
+			$errors->{'BAD_'.$self->{'pkey'}} = 1;
+		}
+	}
+
+	# check each unique column constraint
+	foreach (@{$self->{'unique'}}) {
+		my $res = $dbh->selectall_arrayref("
+			SELECT 
+				1 
+			FROM 
+				$self->{'table'}
+			WHERE 
+				$_ = ?",
+			undef,
+			$values->{$_});
+		if ($res->[0]->[0] == 1) {
+			$errors->{"DUP_$_"} = 1;
+		}
+	}
+
+	return ($values,$errors);
+}
+
+sub validate_edit {
+	my $self   = shift;
+	my $p      = shift;
+
+	my $dbh    = $p->{'dbh'};
+	my $params = $p->{'params'};
+
+	unless ($params->{$self->{'pkey'}} =~ /$self->{'pkey_regexp'}/) {
+		return $self->display_error("Invalid ID");
+	}
+
+	my $errors = {};
+	# call each of the update callbacks
+	foreach (@{$self->{'update_callbacks'}}) {
+		# call back should return a list of error strings
+		my $callback_errors = $_->($dbh,$params);
+		@{$errors}{keys %{$callback_errors}} = values %{$callback_errors};
+	}
+
+	# run the standard error checks
+	my ($values,$e) = $self->{valid}->validate($params);
+
+	# copy the errors from the process_params
+	$errors = { %{$errors}, %{$e} };
+
+	# check all the unique columns
+	foreach (@{$self->{'unique'}}) {
+		my $res = $dbh->selectall_arrayref("
+			SELECT 
+				1
+			FROM 
+				$self->{'table'}
+			WHERE 
+				$_ = ? AND 
+				$self->{'pkey'} != ?",
+			undef,
+			$values->{$_},
+		$params->{$self->{'pkey'}});
+		if ($res->[0]->[0] == 1) {
+			$errors->{"DUP_$_"} = 1;
+		}
+	}
+
+	return $values,$errors;
+}
+
 sub add {
 	my $self = shift;
 	my $p = shift;
@@ -203,64 +398,10 @@ sub add {
 	my $errors = {};
 
 	$self->{'success'} = 0;
+	$self->{'add_details'} = [];
 
 	if ($params->{'cm'} eq "add") {
-		# we're going to attempt the addition
-		my $values;
-
-		# call each of the insert_callbacks
-		foreach (@{$self->{'insert_callbacks'}}) {
-			my $callback_errors = $_->($dbh,$params);
-			@{$errors}{keys %{$callback_errors}} = values %{$callback_errors};
-		}
-
-		my $e;
-
-		# do all the normal parameter checking
-		($values,$e) = $self->{valid}->validate($params);
-
-		if (defined($e)) {
-			# copy the errors from the process_params
-			$errors = { %{$errors}, %{$e} };
-		}
-
-		# check to see if the user supplied primary key (optional) is unique
-		if ($self->{'pkey_user_supplied'}) {
-			if ($params->{$self->{'pkey'}} =~ /$self->{'pkey_regexp'}/) {
-				my $res = $dbh->selectall_arrayref("
-					SELECT 
-						1 
-					FROM 
-						$self->{'table'} 
-					WHERE 
-						$self->{'pkey'} = ?",
-					undef,
-					$params->{$self->{'pkey'}} ) || $self->db_error();
-
-				if ($res->[0]->[0] == 1) {
-					$errors->{'DUP_'.$self->{'pkey'}} = 1;
-				}
-			}
-			else {
-				$errors->{'BAD_'.$self->{'pkey'}} = 1;
-			}
-		}
-
-		# check each unique column constraint
-		foreach (@{$self->{'unique'}}) {
-			my $res = $dbh->selectall_arrayref("
-				SELECT 
-					1 
-				FROM 
-					$self->{'table'}
-				WHERE 
-					$_ = ?",
-				undef,
-				$values->{$_}) || $self->db_error();
-			if ($res->[0]->[0] == 1) {
-				$errors->{"DUP_$_"} = 1;
-			}
-		}
+		my ($values,$errors) = $self->validate_add($p);
 
 		if (scalar keys %{$errors}) {
 			$errors->{'HAS_ERRORS'} = 1;
@@ -280,6 +421,12 @@ sub add {
 			my $q = join(",",map {"?"} @{$self->{'columns'}});		# the ? mark placeholders
 
 			my @v = map { $values->{$_} } @{$self->{'columns'}};	# and the values
+			
+			# store the values as they went into the db here incase the caller wants to
+			# use them for something.
+			foreach (@{$self->{'columns'}}) {
+				push(@{$self->{'add_details'}},[$_,'',$values->{$_}]);
+			}
 
 			if ($self->{'pkey_user_supplied'}) {
 				$c .= ",".$self->{'pkey'};
@@ -291,10 +438,7 @@ sub add {
 
 			my $insert_statement = "INSERT INTO $self->{'table'} ($c) VALUES ($q)";
 
-			$dbh->do($insert_statement,
-			          undef,
-					 @v
-			         ) || $self->db_error();
+			$dbh->do($insert_statement, undef, @v);
 
 			$self->{'success'} = 1;
 			return 1;
@@ -310,7 +454,7 @@ sub add {
 		                $_->{'table'}
 		                $_->{'sextra'}";
 
-		my $res = $dbh->selectall_arrayref($query) || $self->db_error();
+		my $res = $dbh->selectall_arrayref($query);
 
 		$errors->{$_->{'fkey'}} = $self->prep_select($res,$errors->{$_->{'fkey'}} || $_->{'sdefault'});
 	}
@@ -328,9 +472,8 @@ sub edit {
 	$self->{'success'} = 0;
 	$self->{'edit_details'} = [];
 
-	my $dbh       = $p->{'dbh'};
-	my $session   = $p->{'session'};
-	my $params    = $p->{'params'};
+	my $dbh    = $p->{'dbh'};
+	my $params = $p->{'params'};
 
 	# make sure our additional constraint won't break the sql
 	$additional_constraint =~ s/^\s*(where|and|or)\s+//go;
@@ -352,7 +495,7 @@ sub edit {
 			$self->{'pkey'} = ? 
 			$additional_constraint",
 		undef,
-		$params->{$self->{'pkey'}}) || $self->db_error();
+		$params->{$self->{'pkey'}});
 
 	unless (defined($res->[0])) {
 		return $self->display_error("No record with that ID found");
@@ -365,39 +508,7 @@ sub edit {
 
 	my $errors = {};
 	if ($params->{'cm'} eq "update") {
-		my $values;
-
-		# call each of the insert_callbacks
-		foreach (@{$self->{'update_callbacks'}}) {
-			# call back should return a list of error strings
-			my $callback_errors = $_->($dbh,$params);
-			@{$errors}{keys %{$callback_errors}} = values %{$callback_errors};
-		}
-
-		my $e;
-		# run the standard error checks
-		($values,$e) = $self->{valid}->validate($params);
-
-		# copy the errors from the process_params
-		$errors = { %{$errors}, %{$e} };
-
-		# check all the unique columns
-		foreach (@{$self->{'unique'}}) {
-			my $res = $dbh->selectall_arrayref("
-				SELECT 
-					1
-				FROM 
-					$self->{'table'}
-				WHERE 
-					$_ = ? AND 
-					$self->{'pkey'} != ?",
-				undef,
-				$values->{$_},
-				$params->{$self->{'pkey'}}) || $self->db_error();
-			if ($res->[0]->[0] == 1) {
-				$errors->{"DUP_$_"} = 1;
-			}
-		}
+		my ($values,$errors) = $self->validate_edit($p);
 
 		if (scalar keys %{$errors}) {
 			$errors->{'has_errors'} = 1;
@@ -435,7 +546,7 @@ sub edit {
 			$dbh->do($update_statement,
 			          undef,
 			          (map { $values->{$_} } @{$self->{'columns'}}),
-			          $params->{$self->{'pkey'}}) || $self->db_error();
+			          $params->{$self->{'pkey'}});
 
 			$self->{'success'} = 1;
 			return 1;
@@ -468,7 +579,7 @@ sub edit {
 						$_->{'table'}
 						$_->{'sextra'}";
 
-		my $res = $dbh->selectall_arrayref($query) || $self->db_error();
+		my $res = $dbh->selectall_arrayref($query);
 
 		$errors->{$_->{'fkey'}} = $self->prep_select($res,$errors->{$_->{'fkey'}} || $_->{'sdefault'});
 	}
@@ -510,7 +621,7 @@ sub delete {
 			$self->{'pkey'} = ?
 		$additional_constraint",
 		undef,
-		$params->{$self->{'pkey'}}) || $self->db_error();
+		$params->{$self->{'pkey'}});
 
 	unless ($res->[0]->[0] == 1) {
 		return $self->display_error("No Record found with that ID");
@@ -525,7 +636,7 @@ sub delete {
 				$self->{'pkey'} = ?
 			$additional_constraint",
 			undef,
-			$params->{$self->{'pkey'}}) || $self->db_error();
+			$params->{$self->{'pkey'}});
 
 		$self->{'success'} = 2;
 
@@ -554,128 +665,206 @@ sub list {
 	my $dbh    = $p->{'dbh'};
 	my $params = $p->{'params'};
 
+	# hello warning supression
+	$params->{'sort'}      ||= '';
+	$params->{'last_sort'} ||= $params->{'sort'};
+	$params->{'count'}     ||= '';
+	$params->{'page'}      ||= '';
+	$params->{'start'}     ||= '';
+	$params->{'desc'}      ||= '';
+	$params->{'showall'}   ||= '';
+	$params->{'pattern'}   ||= '';
+	$params->{'limit'}     ||= '';
+
 	$params->{'sort'}      =~ s/[^\w-]//g;
 	$params->{'last_sort'} =~ s/[^\w-]//g;
 
 	$params->{'count'}   =~ s/\D//g;
 	$params->{'page'}    =~ s/\D//g;
+	$params->{'start'}   =~ s/\D//g;
 	$params->{'desc'}    =~ s/\D//g;
 	$params->{'showall'} =~ s/\D//g;
 
 	my $pattern = $params->{'pattern'};
 	my $limit   = $params->{'limit'};
 
-	my $sort      = $params->{'sort'}      || $self->{'default_sort'};
-	my $last_sort = $params->{'last_sort'} || $self->{'default_sort'};
-	my $desc      = $params->{'desc'};
-	
-	my $count   = $params->{'count'}   || $self->{'pager'}->{'count'};
-	my $page    = $params->{'page'}    || 1;
+	my $sort;
+	if (defined($self->{'list_sort'}->{$params->{'sort'}})) {
+		$sort = $params->{'sort'};
+	}
+	else {
+		$sort = $self->{'default_sort'};
+	}
+
+	my $last_sort;
+	if (defined($self->{'list_sort'}->{$params->{'last_sort'}})) {
+		$last_sort = $params->{'last_sort'};
+	}
+	else {
+		$last_sort = $self->{'default_sort'};
+	}
+
+	my $desc    = $params->{'desc'};
 	my $showall = $params->{'showall'} || 0;
 
-	# figure out tables to join against
-	my @list;
+	my $count = ($params->{'count'})?$params->{'count'}:$self->{'pager'}->{'count'};
+	my $page  = ($params->{'page'} )?$params->{'page'} :1;
+
+	my $offset = ($params->{'start'})?$params->{'start'}:$count * ($page -1);
+
+	my @search_params = $self->{'list_param_parser'}->($self,$dbh,$params);
+
+	# create the initial list of columns
+	my @columns;
 	foreach ($self->{'pkey'}, @{$self->{'columns'}}) {
 		if ($_ =~ /\./) {
-			push(@list,$_);
+			push(@columns,$_);
 		}
 		else {
-			push(@list,"$self->{'table'}.$_");
+			push(@columns,"$self->{'table'}.$_");
 		}
 	}
 
 	if (ref($additional_constraint)) {
 		if (defined($additional_constraint->{'additional_column'})) {
-			push(@list, $additional_constraint->{'additional_column'});
+			push(@columns, $additional_constraint->{'additional_column'});
 		}
 	}
 
 	# figure out tables to join against
 	my @joins;
-	if ($self->{references}) {
-		foreach my $join ( sort { ($a->{fkey} =~ /\./) <=> ($b->{fkey} =~ /\./) } @{$self->{'references'}}) {
-			if ($join->{fkey} =~ /\./) {
-				push(@joins,"LEFT JOIN $join->{'table'} ON $join->{'fkey'} = $join->{'table'}.$join->{'pkey'}");
-			}
-			else {
-				push(@joins,"LEFT JOIN $join->{'table'} ON $self->{'table'}.$join->{'fkey'} = $join->{'table'}.$join->{'pkey'}");
-			}
-
-			foreach (@{$join->{'columns'}}) {
-				push(@list,"$join->{'table'}.$_");
-			}
-		}
-	}
-
-	if ($self->{joins}) {
-		foreach my $join (@{$self->{joins}}) {
-			my $fkey;
-			if ($join->{fkey} =~ /\./) {
-				$fkey = $join->{fkey};
-			}
-			else {
-				$fkey = $self->{table}.'.'.$join->{fkey};
-			}
+	if ($self->{'references'}) {
+		foreach my $join ( sort { ($a->{'fkey'} =~ /\./) <=> ($b->{'fkey'} =~ /\./) } @{$self->{'references'}}) {
+			my $fkey = ($join->{'fkey'} =~ /\./)?$join->{'fkey'} : $self->{'table'}.'.'.$join->{'fkey'};
 
 			push(@joins,"LEFT JOIN $join->{'table'} ON $fkey = $join->{'table'}.$join->{'pkey'}");
 
-			foreach (@{$join->{columns}}) {
-				if ($_ =~ /\./) {
-					push(@list,$_);
-				}
-				else {
-					push(@list,$join->{'table'}.".$_");
-				}
+			foreach (@{$join->{'columns'}}) {
+				push(@columns,"$join->{'table'}.$_");
 			}
 		}
 	}
 
-	my $select_stmt = "
-		SELECT SQL_CALC_FOUND_ROWS " .
-			join(",\n",@list). "
-		FROM 
-			$self->{'table'} ".
-		join("\n",@joins);
+	foreach my $join (@{$self->{'joins'}},@{$self->{'list_joins'}}) {
+		my @join_clauses = ();
+		my $join_stmt = "$join->{type} JOIN $join->{'table'} ON ";
+
+		if($join->{'pkey'} and $join->{'fkey'}){
+			push(@join_clauses,
+		 		(($join->{'fkey'} =~ /\./) ? $join->{'fkey'} : $self->{'table'} .".". $join->{'fkey'}).
+		 			" = " .
+				(($join->{'pkey'} =~ /\./) ? $join->{'pkey'} : $join->{'table'} .".". $join->{'pkey'})
+			);
+		}
+		
+		if($join->{'extra'}){
+			push(@join_clauses, $join->{'extra'}) unless ref $join->{'extra'};
+			push(@join_clauses, @{$join->{'extra'}}) if ref($join->{'extra'}) eq 'ARRAY';
+		}
+
+		next unless scalar @join_clauses;
+		push(@joins,$join_stmt . join(" AND ", @join_clauses));
+
+		foreach (@{$join->{'columns'}}) {
+			if ($_ =~ /\./) {
+				push(@columns,$_);
+			}
+			else {
+				push(@columns,$join->{'table'}.".$_");
+			}
+		}
+	}
 
-	# make sure our additional constraint won't break the sql
-	$additional_constraint =~ s/^\s*(where|and|or)\s+//go;
 	if (defined($self->{'list_search'}->{$limit}) && $self->safe_text($pattern)) {
-		# we need to narrow the set
-		# 7-18-2001 added lower to make case insensitive for Postgres
-		$select_stmt .= " WHERE $limit LIKE LOWER(\"$pattern%\") ";
-
-		# FIXME!!!
-		if (ref($additional_constraint)) {
-			if(defined($additional_constraint->{'additional_constraint'})) {
-				$select_stmt .= "AND ".$additional_constraint->{'additional_constraint'};
-			}
-		} elsif (length($additional_constraint)) {
-			$select_stmt .= "AND $additional_constraint";
-		}
-	} elsif(ref($additional_constraint)) {
-		if(defined($additional_constraint->{'additional_constraint'})) {
-			$select_stmt .= " WHERE ".$additional_constraint->{'additional_constraint'};
-		}
-
-		# these aren't valid, fry em
-		$pattern = undef;
-		$limit   = undef;
-
-	} elsif (length($additional_constraint)) {
-		$select_stmt .= " WHERE $additional_constraint";
-
-		# these aren't valid, fry em
-		$pattern = undef;
-		$limit   = undef;
+		push(@search_params,[$limit,'LIKE',$pattern]);
 	}
 
+	if ($additional_constraint) {
+		if (ref($additional_constraint) eq "HASH" and
+		    defined($additional_constraint->{'additional_constraint'})) {
+
+			# make sure our additional constraint won't break the sql
+			my $ac = $additional_constraint->{'additional_constraint'};
+			$ac =~ s/^\s*(where|and|or)\s+//go;
+			push(@search_params,$ac);
+		}
+		elsif (!ref($additional_constraint)) {
+			$additional_constraint =~ s/^\s*(where|and|or)\s+//go;
+			push(@search_params,$additional_constraint);
+		}
+	}
+
+	$self->debug(\@search_params);
+	# Make sure the search params are sane
+	my @where;
+	my @values;
+	foreach my $clause (@search_params) {
+		my $r = ref($clause);
+		if ($r eq "ARRAY") {
+			unless ($clause->[0] =~ /\./) {
+				$clause->[0] = $self->{'table'}.'.'.$clause->[0];
+			}
+
+			next unless grep { $clause->[0] } @columns;
+
+			if (scalar(@{$clause}) eq 1) {
+				push(@where,"$r->[0] = 1");
+			}
+			elsif (scalar(@{$clause}) == 3) {
+				if ($clause->[1] =~ /^is(\s+not)?$/i && $clause->[2] =~ /^null$/i) {
+					push(@where,join(" ",@{$clause}));
+				}
+				elsif ($clause->[1] =~ /^(=|!=|>|<|>=|<=)/) {
+					push(@where,"$clause->[0] $clause->[1] ?");
+					push(@values,$clause->[2]);
+				}
+				elsif ($clause->[1] =~ /^(not )?\s*like/i) {
+					if ($dbh->get_info(17) eq "SQLite") {
+						push(@where,"$clause->[0] $clause->[1] ? || '%'");
+					}
+					else {
+						push(@where,"$clause->[0] $clause->[1] concat(?,'%')");
+					}
+					push(@values,$clause->[2]);
+				}
+			}
+		}
+		elsif (!$r) {
+			push(@where,$clause);
+		}
+		else {
+			return $self->exception("each entry in the search params list must either be a scalar or a 3 element array");
+		}
+	}
+
+	my $where = ' ';
+
+	if (scalar(@where)) {
+		$where = "\nWHERE\n".join(" AND\n",@where)."\n";
+	}
+
+	if ($self->{'group_by'}) {
+		$where .= "GROUP BY ".$self->{'group_by'}."\n";
+	}
+
+	# From the DBI docs. This will give us the database server name.
+	my $is_mysql = ($dbh->get_info(17) eq "MySQL")?1:0;
+
+	my $select_stmt = 
+		"SELECT". (($is_mysql)?" SQL_CALC_FOUND_ROWS ": " ").
+		join(",\n",@columns)."\n".
+		"FROM $self->{'table'}\n".
+		join("\n",@joins).
+		$where;
+
+
 	my $n_desc = $desc;
-	if (defined($self->{'list_sort'}->{$sort}) && defined($self->{'list_sort'}->{$last_sort})) {
+	if (defined($sort)) {
 		my $q = $self->{'list_sort'}->{$sort};
 
 		# if we're sorting on the same key as before, then we have the chance to go descending
 		if ($sort eq $last_sort) { 
-			if ($desc == 1) {
+			if ($desc eq '1') {
 				$q =~ s/,/ DESC, /g;
 				$q .= " DESC";
 				$n_desc = 0; # say that we are ascending the next time.
@@ -689,7 +878,7 @@ sub list {
 			$desc = 0;
 		}
 
-		$select_stmt .= " ORDER BY $q";
+		$select_stmt .= "ORDER BY $q\n";
 	}
 	else {
 		# bogus, fry it.
@@ -697,9 +886,19 @@ sub list {
 		$last_sort = undef;
 	}
 
-	$select_stmt .= " LIMIT $count OFFSET ". $count * ($page -1) unless $showall;
+	$select_stmt .= "LIMIT $count OFFSET $offset\n" unless $showall;
 
-	my $page_set = $dbh->selectall_arrayref($select_stmt) || $self->db_error();
+	$self->debug($select_stmt);
+	my $page_set = $dbh->selectall_arrayref($select_stmt,undef,@values);
+
+	my $res_count;
+	if ($is_mysql) {
+		$res_count = $dbh->selectall_arrayref("SELECT FOUND_ROWS()")->[0]->[0];
+	}
+	else {
+		my $count_stmt = "SELECT count(*) FROM $self->{table} ".join("\n",@joins).$where;
+		$res_count = $dbh->selectall_arrayref($count_stmt,undef,@values)->[0]->[0];
+	}
 
 	my %return;
 
@@ -716,8 +915,6 @@ sub list {
 	$return{'LIMIT'}   = $self->prep_select($self->{'list_search_items'},$limit);
 	$return{'PATTERN'} = $pattern;
 
-	my $rc = $dbh->selectall_arrayref("SELECT FOUND_ROWS()");
-	my $res_count = $rc->[0]->[0];
 
 	$return{'NUM_MATCHES'} = $res_count;
 
@@ -726,18 +923,18 @@ sub list {
 	################################################################################
 	my %dates;
 	foreach (@{$self->{'dates'}}) {
-		$dates{$_->{'name'}} = 1;
+		$dates{$_} = 1;
 	}
 
 	my %times;
 	foreach (@{$self->{'times'}}) {
-		$times{$_->{'name'}} = 1;
+		$times{$_} = 1;
 	}
 
 	foreach (@{$page_set}) {
 		my %v;
-		for (my $i=0; $i <= @list; $i++) {
-			my $key = $list[$i];
+		for (my $i=0; $i < @columns; $i++) {
+			my $key = $columns[$i];
 
 			$key =~ s/$self->{'table'}\.//; # take of the table name in front
 			# we either end up with the column name from the primay table,
@@ -795,25 +992,32 @@ sub view {
 		}
 	}
 
-	if ($self->{joins}) {
-		foreach my $join (@{$self->{joins}}) {
-			my $fkey;
-			if ($join->{fkey} =~ /\./) {
-				$fkey = $join->{fkey};
+	foreach my $join (@{$self->{joins}},@{$self->{view_joins}}) {
+		my @join_clauses = ();
+		my $join_stmt = "$join->{type} JOIN $join->{'table'} ON ";
+
+		if($join->{'pkey'} and $join->{'fkey'}){
+			push(@join_clauses,
+		 		(($join->{'fkey'} =~ /\./) ? $join->{'fkey'} : $self->{'table'} .".". $join->{'fkey'}).
+		 			" = " .
+				(($join->{'pkey'} =~ /\./) ? $join->{'pkey'} : $join->{'table'} .".". $join->{'pkey'})
+			);
+		}
+		
+		if($join->{'extra'}){
+			push(@join_clauses, $join->{'extra'}) unless ref $join->{'extra'};
+			push(@join_clauses, @{$join->{'extra'}}) if ref($join->{'extra'}) eq 'ARRAY';
+		}
+
+		next unless scalar @join_clauses;
+		push(@joins,$join_stmt . join(" AND ", @join_clauses));
+
+		foreach (@{$join->{columns}}) {
+			if ($_ =~ /\./) {
+				push(@list,$_);
 			}
 			else {
-				$fkey = $self->{table}.'.'.$join->{fkey};
-			}
-
-			push(@joins,"LEFT JOIN $join->{'table'} ON $fkey = $join->{'table'}.$join->{'pkey'}");
-
-			foreach (@{$join->{columns}}) {
-				if ($_ =~ /\./) {
-					push(@list,$_);
-				}
-				else {
-					push(@list,$join->{'table'}.".$_");
-				}
+				push(@list,$join->{'table'}.".$_");
 			}
 		}
 	}
@@ -828,7 +1032,8 @@ sub view {
 			$self->{'table'}.$self->{'pkey'} = ?
 			$additional_constraint";
 
-	my $res = $dbh->selectall_arrayref($select_statement,undef,$params->{$self->{'pkey'}}) || $self->db_error();
+	#$self->debug($select_statement);
+	my $res = $dbh->selectall_arrayref($select_statement,undef,$params->{$self->{'pkey'}});
 
 	my %v;
 	if (defined($res) && defined($res->[0])) {
@@ -887,7 +1092,7 @@ sub toggle {
 		WHERE 
 			$self->{'pkey'} = ?",
 		undef,
-		$params->{$self->{'pkey'}}) || $self->db_error();
+		$params->{$self->{'pkey'}});
 
 	$self->{'success'} = 1;
 	return 1;
